'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _VER  --> _VER.CMD       |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2012,2013 "Cluso99" (Ray Rodrick)                     |
'' |  Modifications:                                                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' This OS module uses an included module "__MODULE.SPIN" to perform most of
''  the housekeeping - see this for history, details and acknowledgements.
''
'' This module......
'' RR20120317     Displays various system info.
'' RR20151215     add p8xblade & v0.8x
'' RR20151220     add cpublade7


'#include "__MODULE.spin"        ' include the common code for OS modules 
#include "__MODULE.spin"

DAT
_ModuleStr    byte      "=== VER ===",0        'module's name (string)
 
PRI executeCommand(stringPointer) | z,z0,z1,z2,z3,z4,i,j,cogs[8]

'' ifnot(str.stringCompareCI(string("ver"), stringPointer))    '<-- module's name
    Print.CRLF
    Print.StrCR(string("*** Cluso's Propeller Operating System v1.0x ***"))
    Print.StrCR(string("------------------------------------------------"))
    Print.StrCR(string(" Originally derived from KyeDOS...                "))
    Print.StrCR(string("     by Kwabena W. Agyeman & J. Moxham & Cluso99  "))
    Print.StrCR(string("     & acknowledgements to M. Park (Sphinx) etc   "))
    Print.CRLF

    Print.Str(string(" Hardware: "))
    case long[_hub#_hardware] >> 16  'hardware >> 29
      $0001 : Print.Str(string("Cluso's "))
      other : Print.Str(string("??? "))
    case long[_hub#_hardware]  'hardware >> 29
      $0001_0001 : Print.Str(string("RamBlade"))
      $0001_0002 : Print.Str(string("TriBlade#2"))
      $0001_0003 : Print.Str(string("RamBlade3"))
      $0001_0004 : Print.Str(string("P8XBlade2"))
      $0001_0007 : Print.Str(string("CpuBlade7"))
      other: Print.Str(string("???"))
    Print.CRLF

    Print.Str(string(" ClockFreq "))
    Print.Str(1 + str.integerToDecimal(long[$0],9))
    Print.Str(string("Hz, ClockMode $"))
    Print.Str(str.integerToHexadecimal(byte[$4],2))
    Print.Str(string(", Cog "))
    Print.StrCR(1 + str.integerToDecimal(cogid, 1))

    Print.Str(string(" SD card on pins "))
    z  := long[_hub#_sdPins]
    z0 :=  z        & $1F
    z1 := (z >>  8) & $1F
    z2 := (z >> 16) & $1F
    z3 := (z >> 24) & $1F
    Print.Str(1 + str.integerToDecimal(z0, 2))
    Print.Char(",")
    Print.Str(1 + str.integerToDecimal(z1, 2))
    Print.Char(",")
    Print.Str(1 + str.integerToDecimal(z2, 2))
    Print.Char(",")
    Print.Str(1 + str.integerToDecimal(z3, 2))
    Print.CRLF

    Print.Str(string(" Serial    on pins "))
    z  := long[_hub#_sioPins]
    z0 :=  z        & $1F
    z1 := (z >>  8) & $1F
    z2 := (z >> 16) & $1F
    z3 := (z >> 24) & $0F                               ' =$0F if invalid
    z4 :=  long[_hub#_sioBaud]
    Print.Str(1 + str.integerToDecimal(z0, 2))
    Print.Char(",")
    Print.Str(1 + str.integerToDecimal(z1, 2))
    Print.Str(string(", Mode "))
    Print.Str(1 + str.integerToDecimal(z2, 2))
    Print.Str(string(", Baud "))
    Print.Str(1 + str.integerToDecimal(z4, 8))
    Print.Str(string(", Cog "))
    if z3 > 7
      Print.Char("?")
    else
      Print.Str(1 + str.integerToDecimal(z3, 1))
    Print.CRLF

    Print.Str(string(" Screen size: "))
    z := byte[_hub#_OS_Columns]
    Print.Str(1 + str.integerToDecimal(z, 3))
    Print.Str(string(" x "))
    z := byte[_hub#_OS_Rows]
    Print.StrCR(1 + str.integerToDecimal(z, 3))

    Print.Str(string(" <LF> is "))
    if byte[_hub#_OS_Rows] & _hub#_LF_MASK                                              
      Print.StrCR(string("ON"))
    else
      Print.StrCR(string("OFF"))

    Print.Str(string(" Resident Cogs : "))
    repeat z from 0 to 7
      if (byte[_hub#_OS_Cogs] >> z) & $01 == 1
        Print.Str(1 + str.integerToDecimal(z, 1))
        Print.Str(string(" "))
    Print.Str(string("    This Cog: "))
    Print.StrCR(1 + str.integerToDecimal(cogid, 1))

    Print.Str(string(" Cogs available: "))
    j~
    repeat i from 0 to 7
      cogs[i] := cognew(@entry,0) +1                    ' try to start cog(s)
      if cogs[i] 
        j++
    repeat i from 0 to 7
      if cogs[i]
        Print.Str(1 + str.integerToDecimal(cogs[i]-1, 1)) ' print avail cog#
        Print.Char(" ")
        cogstop(cogs[i] -1)                             ' stop the cog(s)
    Print.Str(string("(="))
    Print.Str(1 + str.integerToDecimal(j,1))
    Print.StrCR(string(")"))

    Print.CRLF
    return 0

DAT
'' Just a simple pasm program to test if a cog is available
              org       0
entry         jmp       #entry                  ' loops here until cogstop forced!

dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}